package com.persagy.dmp.fss.ioc.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.fss.common.service.BaseFileService;
import com.persagy.dmp.fss.common.service.FileService;
import com.persagy.dmp.fss.file.entity.FileResource;
import com.persagy.dmp.fss.ioc.config.properties.IOCProperties;
import com.persagy.dmp.fss.ioc.tool.HttpTools;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.LinkedHashMap;
import java.util.Map;

@Service
public class IOCFileService extends BaseFileService implements FileService {

    @Autowired
    private IOCProperties iocProperties;

    /**
     * 存储 显示调用父类使事务生效
     * @param fileName
     * @param appId
     * @param userId
     * @param groupCode
     * @param projectId
     * @param file
     * @return
     * @throws Exception
     */
    @Transactional(propagation= Propagation.REQUIRED,rollbackFor=Exception.class)
    @Override
    public String upload(String fileName, String appId, String userId, String groupCode, String projectId, MultipartFile file) throws Exception {
        return super.upload(fileName, appId, userId, groupCode, projectId, file);
    }

    /**
     * 存储实现
     * @param fileName
     * @param appId
     * @param groupCode
     * @param projectId
     * @param file
     * @return
     * @throws Exception
     */
    @Override
    protected ObjectNode store(String fileName, String appId, String groupCode, String projectId, MultipartFile file) throws Exception {
        // 获取Token
        Map<String,String> map = new LinkedHashMap<>();
        map.put("accessKey", iocProperties.getAccessKey());
        map.put("secretKey", iocProperties.getSecretKey());
        map.put("bucketName", iocProperties.getBucketName());
        map.put("systemUrl", iocProperties.getSystemUrl());
        String json = objectMapper.writeValueAsString(map);

        String tokenUrl = iocProperties.getSystemUrl() + iocProperties.getTokenUrl();
        String result = HttpTools.postJson(tokenUrl,json);

        Map resultMap = objectMapper.readValue(result, Map.class);
        String code = (String) resultMap.get("code");
        if(!"Y".equals(code)){
            throw new Exception(result);
        }

        // 上传文件
        String authorization = (String) resultMap.get("body");
        String uploadUrl = iocProperties.getSystemUrl() + iocProperties.getUploadUrl();
        String upload = HttpTools.uploadFile(file.getBytes(), fileName, iocProperties.getUserId(), authorization, uploadUrl);
        resultMap = objectMapper.readValue(upload, Map.class);
        code = (String) resultMap.get("code");
        if(!"Y".equals(code)){
            throw new Exception(upload);
        }
        ObjectNode objectNode = objectMapper.createObjectNode();
        objectNode.putPOJO("ioc", resultMap);

        return objectNode;
    }

    /**
     * 将存储转换为二进制数组
     * @param fileResource
     * @return
     * @throws Exception
     */
    @Override
    public byte[] download(FileResource fileResource) throws Exception {
        ObjectNode extension = fileResource.getExtension();
        JsonNode body = extension.get("ioc").get("body");
        String bucketName = body.get("bucketName").asText();
        String resourceId = body.get("resourceId").asText();
        String resourceName = body.get("resourceName").asText();

        StringBuffer downloadUrl = new StringBuffer(iocProperties.getSystemUrl());
        downloadUrl.append(iocProperties.getDownloadUrl());
        downloadUrl.append("/");
        downloadUrl.append(bucketName);
        downloadUrl.append("/");
        downloadUrl.append(resourceId);
        downloadUrl.append("/");
        downloadUrl.append(resourceName);

        byte[] file = HttpTools.downloadFile(downloadUrl.toString());
        return file;
    }

    /**
     * 删除存储 显示调用父类使事务生效
     * @param resourceId
     * @throws Exception
     */
    @Transactional(propagation= Propagation.REQUIRED,rollbackFor=Exception.class)
    @Override
    public void delete(String resourceId) throws Exception {
        super.delete(resourceId);
    }

    /**
     * 清除实现
     * @param fileResource
     * @throws Exception
     */
    @Override
    protected void clear(FileResource fileResource) throws Exception {
        Map<String,String> map = new LinkedHashMap<>();
        map.put("accessKey", iocProperties.getAccessKey());
        map.put("secretKey", iocProperties.getSecretKey());
        map.put("bucketName", iocProperties.getBucketName());

        String json = objectMapper.writeValueAsString(map);

        String tokenUrl = iocProperties.getSystemUrl() + iocProperties.getDeleteTokenUrl();
        String result = HttpTools.postJson(tokenUrl,json);

        Map resultMap = objectMapper.readValue(result, Map.class);
        String code = (String) resultMap.get("code");
        if(!"Y".equals(code)){
            throw new Exception(result);
        }

        // 上传文件
        String authorization = (String) resultMap.get("body");
        JsonNode body = fileResource.getExtension().get(this.getType()).get("body");
        String resourceId = body.get("resourceId").asText();

        String deleteUrl = iocProperties.getSystemUrl() + iocProperties.getRemoveUrl() + "/" + resourceId;
        String delete = HttpTools.delete(deleteUrl, authorization);
        resultMap = objectMapper.readValue(delete, Map.class);
        code = (String) resultMap.get("code");
        if(!"Y".equals(code)){
            throw new Exception(delete);
        }
    }

    /**
     * 文件存储类型
     * @return
     */
    @Override
    protected String getType() {
        return "ioc";
    }
}
